/******************************************************************
 * Copyright and Permission Notice                                *
 *                                                                *
 * Permission is granted to copy, use, modify, and merge this     *
 * software into your applications and to permit others to do any *
 * of the foregoing for non-commercial use only. You must include *
 * this permission and copyright notice in all copies and         *
 * modified versions of this software, and include attribution    *
 * to Taligent in all splash screens included in any application  *
 * using this software. You must require others to whom you       *
 * distribute this software to do likewise.                       *
 *                                                                *
 * To obtain an unrestricted commercial license to the software   *
 * source and/or binaries, please contact Taligent at             *
 * widgets@taligent.com                                           *
 *                                                                *
 * THE SOFTWARE IS PROVIDED IN ITS 'AS IS' CONDITION FOR          *
 * NON-COMMERCIAL USE ONLY. TALIGENT DISCLAIMS ANY LIABILITY OF   *
 * ANY KIND FOR DAMAGES WHATSOEVER RESULTING FROM THE USE OF      *
 * THIS SOFTWARE.                                                 *
 *                                                                *
 * (C) Copyright Taligent, Inc. 1997-All Rights Reserved.         *
 * (C) Copyright IBM Corporation 1997                             *
 ******************************************************************/

import java.awt.*;
import java.awt.event.*;

import COM.taligent.widget.*;
import COM.taligent.widget.event.*;

/**
 * A simple applet to demonstrate the features of the
 * MaskedTextField component.
 *
 * @author  Andy Clark, Taligent Inc.
 * @version 1.0
 */
public class MaskedTextFieldApplet
    extends java.applet.Applet
    implements ItemListener,
               ActionListener,
               WindowListener,
               TextListener,
               MaskedTextFieldListener
    {
    //
    // GUI Components
    //
    private Panel           example = new BorderPanel("Example");
    private Choice          mask    = new Choice();
    private TextField       enter   = new TextField();
    private MaskedTextField text    = new MaskedTextField();
    private Checkbox        show    = new Checkbox("Show Invalid");

    private Panel    settings = new BorderPanel("Settings");
    private Choice   dir      = new Choice();
    private Checkbox invalid  = new Checkbox("Allow invalid input");
    private Checkbox shift    = new Checkbox("Shift text");
    private Checkbox move     = new Checkbox("Move cursor on input");
    private Button   edit     = new Button("Edit Mask Characters");

    private Dialog dialog;
    private Button done;

    //
    // Data
    //
    private int lastselected;

    // setup applet
    public void init() {

        setBackground(Color.lightGray);
        setLayout(new GridLayout(1, 2, 2, 2));

        // setup example panel
        String masks[] = {
                "[ add custom mask ]",
                "(###) ###-####",
                "###-##-####",
                "##/##/##",
                "\\#AA aa ## @@"
                };
        for (int i = 0; i < masks.length; i++)
            mask.addItem(masks[i]);

        lastselected = 1;
        mask.select(lastselected);
        text.setMask(masks[lastselected]);

        example.setLayout(new GridLayout(5, 1, 2, 2));

        example.add(new Label("Sample mask:"));
        example.add(mask);
        example.add(new Label("Try the masked input:"));
        example.add(text);
        example.add(show);

        add(example);

        // setup settings panel
        settings.setLayout(new GridLayout(5, 1, 2, 2));

        Panel dirpanel = new Panel();
        dirpanel.setLayout(new GridLayout(1, 2, 2, 2));

        dirpanel.add(new Label("Direction:"));
        dirpanel.add(dir);

        settings.add(dirpanel);
        settings.add(invalid);
        settings.add(shift);
        settings.add(move);
        settings.add(edit);

        add(settings);

        // set values
        show.setState(true);

        InputDirection dirvalue = text.getInputDirection();
        for (int i = 0; i < InputDirection.TAGS.length; i++) {
            dir.addItem(InputDirection.TAGS[i]);
            if (dirvalue.equals(InputDirection.DIRECTIONS[i]))
                dir.select(i);
            }

        invalid.setState(text.getAllowInvalid());
        shift.setState(text.getShiftText());
        move.setState(text.getMoveCursor());

        // add listeners
        mask.addItemListener(this);
        enter.addActionListener(this);

        text.addActionListener(this);
        text.addTextListener(this);
        text.addMaskedTextFieldListener(this);

        show.addItemListener(this);

        dir.addItemListener(this);
        invalid.addItemListener(this);
        shift.addItemListener(this);
        move.addItemListener(this);
        edit.addActionListener(this);

        setSize(330, 150);
        setVisible(true);
        }

    //
    // ItemEvent methods
    //

    /** A new mask was selected. */
    public void itemStateChanged(ItemEvent evt) {
        Object source = evt.getSource();

        // a new mask was selected
        if (source == mask) {
            int index = mask.getSelectedIndex();
            if (index == 0) {
                enter.setText("");

                example.remove(mask);

                // Note: The following line is dependent on
                // the way that I've set up the panel. -Ac
                example.add(enter, 1);

                example.validate();
                example.repaint();

                enter.requestFocus();
                }
            else {
                lastselected = index;
                text.setMask(mask.getItem(lastselected));
                }
            }

        // show invalidness
        else if (source == show) {
            if (show.getState()) {
                if (!text.validateText())
                    text.setBackground(Color.red);
                }
            else {
                text.setBackground(Color.white);
                }
            }

        // input direction
        else if (source == dir) {
            int index = dir.getSelectedIndex();
            InputDirection input = InputDirection.DIRECTIONS[index];
            text.setInputDirection(input);
            }

        // allow invalid input
        else if (source == invalid) {
            text.setAllowInvalid(invalid.getState());
            }

        // shift text
        else if (source == shift) {
            text.setShiftText(shift.getState());
            }

        // move cursor
        else if (source == move) {
            text.setMoveCursor(move.getState());
            }
        }

    //
    // ActionEvent methods
    //

    /** An action was performed on the text field. */
    public void actionPerformed(ActionEvent evt) {
        Object source = evt.getSource();

        if (source == text) {
            // nothing
            }
        else if (source == edit) {
            // create dialog box
            if (dialog == null) {
                dialog = new Dialog(getFrame(this),
                                    "Edit Mask Characters",
                                    true // modal
                                    );

                // setup dialog
                MaskedCustomizer customizer = new MaskedCustomizer();
                customizer.setObject(text);

                done = new Button("Done");

                dialog.setBackground(Color.lightGray);
                dialog.setLayout(new BorderLayout());

                dialog.add("Center", customizer);
                dialog.add("South",  done);

                // add listeners
                dialog.addWindowListener(this);
                done.addActionListener(this);

                // set size
                dialog.setSize(240, 230);
                }

            // show dialog
            dialog.setVisible(true);
            }
        else if (source == done) {
            dialog.setVisible(false);
            }
        else if (source == enter) {
            String str = enter.getText();
            if (str.length() > 0) {
                // add new mask to list
                mask.addItem(str);

                lastselected = mask.getItemCount() - 1;
                mask.select(lastselected);

                // set text mask
                text.setMask(str);
                }
            else {
                mask.select(lastselected);
                }

            // switch back to choice
            example.remove(enter);

            // Note: The following line is dependent on
            // the way that I've set up the panel. -Ac
            example.add(mask, 1);

            example.validate();
            example.repaint();
            }
        }

    //
    // WindowEvent methods
    //

    // not used
    public void windowOpened(WindowEvent evt) {}
    public void windowClosed(WindowEvent evt) {}
    public void windowIconified(WindowEvent evt) {}
    public void windowDeiconified(WindowEvent evt) {}
    public void windowActivated(WindowEvent evt) {}
    public void windowDeactivated(WindowEvent evt) {}

    /** Window closed. */
    public void windowClosing(WindowEvent evt) {
        Object source = evt.getSource();

        if (source == dialog) {
            dialog.setVisible(false);
            }
        }

    //
    // TextEvent methods
    //

    /** Text value changed. */
    public void textValueChanged(TextEvent evt) {
        Object source = evt.getSource();

        if (source == text) {
            // nothing
            }
        }

    //
    // MaskedTextFieldEvent methods
    //

    /** The current text input has made the text valid. */
    public void textValidated(MaskedTextFieldEvent evt) {
        Object source = evt.getSource();

        if (source == text) {
            if (show.getState()) {
                text.setBackground(Color.white);
                }
            }
        }

    /** The current text input has made the text invalid. */
    public void textInvalidated(MaskedTextFieldEvent evt) {
        Object source = evt.getSource();

        if (source == text) {
            if (show.getState()) {
                text.setBackground(Color.red);
                }
            }
        }

    //
    // Utility methods
    //

    /**
     * Returns the Frame that the specified component is in.
     *
     * @param comp  The component contained in the Frame.
     */
    public static Frame getFrame(Component comp) {

        // climb the parent ladder to find Frame
        Component parent = comp;
        while (parent != null) {
            if (parent instanceof Frame)
                return (Frame)parent;

            parent = parent.getParent();
            }

        return null;
        }

    }