/******************************************************************
 * Copyright and Permission Notice                                *
 *                                                                *
 * Permission is granted to copy, use, modify, and merge this     *
 * software into your applications and to permit others to do any *
 * of the foregoing for non-commercial use only. You must include *
 * this permission and copyright notice in all copies and         *
 * modified versions of this software, and include attribution    *
 * to Taligent in all splash screens included in any application  *
 * using this software. You must require others to whom you       *
 * distribute this software to do likewise.                       *
 *                                                                *
 * To obtain an unrestricted commercial license to the software   *
 * source and/or binaries, please contact Taligent at             *
 * widgets@taligent.com                                           *
 *                                                                *
 * THE SOFTWARE IS PROVIDED IN ITS 'AS IS' CONDITION FOR          *
 * NON-COMMERCIAL USE ONLY. TALIGENT DISCLAIMS ANY LIABILITY OF   *
 * ANY KIND FOR DAMAGES WHATSOEVER RESULTING FROM THE USE OF      *
 * THIS SOFTWARE.                                                 *
 *                                                                *
 * (C) Copyright Taligent, Inc. 1997-All Rights Reserved.         *
 * (C) Copyright IBM Corporation 1997                             *
 ******************************************************************/

import java.awt.*;
import java.awt.event.*;

import COM.taligent.widget.*;
import COM.taligent.widget.event.*;

/**
 * Sample applet that illustrates some of the most common
 * ways to use the MaskedTextField.
 *
 * @author  Andy Clark, Taligent Inc.
 * @version 1.0
 */
public class MaskedTextFieldExamples
    extends java.applet.Applet
    implements ItemListener,
               TextListener,
               ActionListener
    {
    //
    // Constants
    //
    private static final String SHOW_CODE = "Code >>>";
    private static final String HIDE_CODE = "Code <<<";

    private static final int SPECIAL_CASE = 4;

    // text field settings
    private static final String DESC[] = {
        "Phone Number",
        "Social Security Number",
        "4 Digit Serial Number",
        "Integer: 0-9999",
        "Text Only with Spaces",
        "American Currency Input"
        };
    private static final String MASK[] = {
        "(###) ###-####",           "###-##-####",
        "####",                     "####",
        "@@@@@@@@@@@@@@@@@@@@",     "$####.##"
        };
    private static final InputDirection DIR[] = {
        InputDirection.RIGHT,   InputDirection.RIGHT,
        InputDirection.RIGHT,   InputDirection.LEFT,
        InputDirection.RIGHT,   InputDirection.LEFT
        };
    private static final boolean INVALID[] = {
        false,  false,  false,  false,  false,  false
        };
    private static final boolean SHIFT[] = {
        true,   true,   true,   true,   true,   true
        };
    private static final boolean MOVE[] = {
        true,   true,   true,   false,  true,   false
        };
    private static final char BLANKCHAR[] = {
        '0', '0', '0',
        MaskedTextField.DEFAULT_BLANK_CHAR,
        ' ',
        MaskedTextField.DEFAULT_BLANK_CHAR
        };
    private static final boolean RETURNBLANKS[] = {
        true,   true,   true,   false,  false,  false
        };

    // sample source code
    private static final String CODE[] = {
        "// Standard phone number\n"+
        "MaskedTextField mask = new MaskedTextField(\"(###) ###-####\")\n"+
        "\n"+
        "mask.setBlankChar('0');\n"+
        "mask.setReturnBlanks(true);\n"
        ,

        "// Social Security number\n"+
        "MaskedTextField mask = new MaskedTextField(\"###-##-####\")\n"+
        "\n"+
        "mask.setBlankChar('0');\n"+
        "mask.setReturnBlanks(true);\n"
        ,

        "// A 4 digit serial number, complete entry required\n"+
        "MaskedTextField mask = new MaskedTextField(\"####\")\n"+
        "\n"+
        "mask.setBlankChar('0');\n"+
        "mask.setReturnBlanks(true);\n"
        ,

        "// An integer from 0-9999, complete entry not required\n"+
        "MaskedTextField mask = new MaskedTextField(\"####\")\n"+
        "\n"+
        "mask.setInputDirection(InputDirection.LEFT);\n"+
        "mask.setMoveCursor(false);\n"
        ,

        "// Text only input, spaces allowed\n"+
        "MaskedTextField mask = new MaskedTextField(\"@@@@@@@@@@@@@@@@@@@@\")\n"+
        "\n"+
        "MaskCharacter maskchars[] = { new MaskCharacter('@', \"A-Za-z \") };\n"+
        "mask.setMaskCharacters(maskchars);\n"+
        "\n"+
        "mask.setBlankChar(' ');\n"
        ,

        "// American currency input\n"+
        "MaskedTextField mask = new MaskedTextField(\"$####.##\")\n"+
        "\n"+
        "mask.setInputDirection(InputDirection.LEFT);\n"+
        "mask.setMoveCursor(false);\n"
        };

    private static final MaskedTextField TEXTFIELD[] = {
        new MaskedTextField(),  new MaskedTextField(),
        new MaskedTextField(),  new MaskedTextField(),
        new MaskedTextField(),  new MaskedTextField()
        };

    private static final Dimension SIZE  = new Dimension(420, 180);
    private static final Dimension SIZE2 = new Dimension(420, 360);

    //
    // GUI Components
    //
    private Choice examples = new Choice();
    private Panel  panel    = new Panel();
    private Label  label    = new Label();

    private Choice   dir     = new Choice();
    private Checkbox invalid = new Checkbox();
    private Checkbox shift   = new Checkbox();
    private Checkbox move    = new Checkbox();

    private Button   show = new Button(SHOW_CODE);
    private TextArea code = new TextArea();

    //
    // Data
    //
    private MaskedTextField currentmask;

    private boolean shown = false;

    /** Setup applet. */
    public void init() {

        // initialize text fields
        for (int i = 0; i < TEXTFIELD.length; i++) {
            MaskedTextField mask = TEXTFIELD[i];

            // set values
            mask.setMask(MASK[i]);
            mask.setInputDirection(DIR[i]);
            mask.setAllowInvalid(INVALID[i]);
            mask.setShiftText(SHIFT[i]);
            mask.setMoveCursor(MOVE[i]);
            mask.setBlankChar(BLANKCHAR[i]);
            mask.setReturnBlanks(RETURNBLANKS[i]);

            // special case
            if (i == SPECIAL_CASE) {
                MaskCharacter maskchars[] = {
                    new MaskCharacter('@', "A-Za-z ")
                    };
                mask.setMaskCharacters(maskchars);
                }

            // add example to list of choices
            examples.addItem(DESC[i]);

            // add listener
            mask.addTextListener(this);
            }

        // initialize components
        for (int i = 0; i < InputDirection.TAGS.length; i++)
            dir.addItem(InputDirection.TAGS[i]);

        dir.setEnabled(false);
        invalid.setEnabled(false);
        shift.setEnabled(false);
        move.setEnabled(false);

        Font font = new Font("Courier", Font.PLAIN, 12);
        show.setFont(font);

        code.setFont(font);
        code.setEditable(false);

        // add listeners
        examples.addItemListener(this);

        show.addActionListener(this);

        // setup display
        setBackground(Color.lightGray);
        setLayout(new BorderLayout());

        Panel north = new Panel();
        north.setLayout(new BorderLayout());

        Panel northCenter = new BorderPanel("Examples");
        northCenter.setLayout(new GridLayout(3, 1, 2, 2));

        northCenter.add(examples);

        panel.setLayout(new BorderLayout());
        northCenter.add(panel);

        northCenter.add(label);

        north.add("Center", northCenter);

        Panel northEast = new BorderPanel("Current Settings");
        northEast.setLayout(new BorderLayout());

        Panel p1 = new Panel();
        p1.setLayout(new GridLayout(4, 1, 2, 2));

        p1.add(new Label("Direction:"));
        p1.add(new Label("Allow invalid input:"));
        p1.add(new Label("Shift text:"));
        p1.add(new Label("Move cursor on input:"));

        northEast.add("West", p1);

        Panel p2 = new Panel();
        p2.setLayout(new GridLayout(4, 1, 2, 2));

        p2.add(dir);
        p2.add(invalid);
        p2.add(shift);
        p2.add(move);

        northEast.add("East", p2);

        north.add("East", northEast);

        Panel northSouth = new Panel();

        northSouth.add(show);

        north.add("South", northSouth);

        add("North", north);

        // set mask
        setMaskedTextField(0);

        // resize and show
        setSize(SIZE);
        setVisible(true);
        }

    /** Set masked text field. */
    public void setMaskedTextField(int index) {

        currentmask = TEXTFIELD[index];

        // exchange components
        panel.removeAll();
        panel.add("Center", currentmask);
        panel.validate();
        panel.repaint();

        // change settings
        if (currentmask.getInputDirection().equals(InputDirection.DIRECTIONS[0]))
            dir.select(0);
        else
            dir.select(1);
        invalid.setState(currentmask.getAllowInvalid());
        shift.setState(currentmask.getShiftText());
        move.setState(currentmask.getMoveCursor());

        // reset label
        setLabel(currentmask.getText());

        // set code
        code.setText(CODE[index]);
        }

    /** Set label. */
    public void setLabel(String text) {
        StringBuffer str = new StringBuffer();

        str.append("text = \"");
        str.append(text);
        str.append('\"');

        label.setText(str.toString());
        }

    //
    // Item listener method
    //
    public void itemStateChanged(ItemEvent evt) {

        setMaskedTextField(examples.getSelectedIndex());
        }

    //
    // Text listener method
    //
    public void textValueChanged(TextEvent evt) {

        setLabel(currentmask.getText());
        }

    //
    // Action listener method
    //
    public void actionPerformed(ActionEvent evt) {

        if (shown) {
            show.setLabel(SHOW_CODE);
            remove(code);
            setSize(SIZE);
            }
        else {
            show.setLabel(HIDE_CODE);
            add("South", code);
            setSize(SIZE2);
            }

        shown = !shown;
        }

    }